/**
 * zmia.cpp
 *
 * author: krasimir.georgiev
 */

#include <cstdio>
using namespace std;

const int max_n = (1 << 19);

struct arrow {
	int x, y;
	char dir;

	arrow(int x = 0, int y = 0, char dir = 0) :
			x(x), y(y), dir(dir) {
	}

	static void rot(int& x, int& y, char dir) {
		int old_x = x;
		switch (dir) {
		case 1:
			x = -y;
			y = old_x;
			break;
		case 2:
			x = -x;
			y = -y;
			break;
		case 3:
			x = y;
			y = -old_x;
			break;
		}
	}

	arrow operator *(const arrow& b) const {
		int nx = b.x;
		int ny = b.y;
		rot(nx, ny, dir);
		nx += x;
		ny += y;

		return arrow(nx, ny, (dir + b.dir) % 4);
	}
};

arrow int_tree[2 * max_n];

void add(int i, char dir) {
	i += max_n;
	int_tree[i] = arrow(0, 0, dir) * int_tree[i];
	for (i /= 2; i > 0; i /= 2) {
		int_tree[i] = int_tree[2 * i] * int_tree[2 * i + 1];
	}
}

arrow get(int i) {
	arrow s(0, 0, 0);
	for (i += max_n; i > 1; i /= 2) {
		if (i % 2) {
			s = int_tree[i - 1] * s;
		}
	}

	return s;
}

int main() {
	int n, q;
	scanf("%d%d", &n, &q);

	for (int i = 0; i < n; ++i) {
		int_tree[max_n + i] = arrow(1, 0, 0);
	}

	for (int i = max_n - 1; i > 0; --i) {
		int_tree[i] = int_tree[2 * i] * int_tree[2 * i + 1];
	}

	while (q--) {
		char c;
		int i;
		scanf(" %c %d", &c, &i);
		if (c == 'L') {
			arrow a = get(i);
			printf("%d %d\n", a.x, a.y);
		} else {
			char d;
			scanf(" %c", &d);
			add(i, d == '+' ? 1 : 3);
		}
	}

	return 0;
}

