/*
ID: espr1t
LANG: C++
TASK: tetris
KEYWORDS: NP, Backtrack
*/

#include <ctime>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <algorithm>

#define MAX 131072
#define INF 1000000000

#define MAX_F 4
#define MAX_L 8
#define MAX_H 32
#define MAX_W 32
#define BOARD_H 20
#define BOARD_W 10

// Set one second under TL
#define MAX_EXEC_TIME 3000

int n;
int ans[MAX][2];
int inpFigs[MAX];
char board[MAX_H][MAX_W];
char tboard[MAX_L][MAX_H][MAX_W];

// Score for intruducing a hole
int holeScore = -100;
// Score for a square at some height
// Actual score is row * heightScore
int heightScore = -10;
int maxDepth = 1;

// Copy from this point onward
// if you want to use in solution
// -->

#include <cstdio>
#include <cstring>
#include <algorithm>

#define BOARD_W 10
#define BOARD_H 20

#define MAX_FIG_SIZE 4
#define MAX_BOARD_SIZE 32

char figs[7][MAX_FIG_SIZE + 1][MAX_FIG_SIZE + 1] = {
	 {" #  ",
	  " #  ",
	  " #  ",
	  " #  "},

	 {" ## ",
	  " #  ",
	  " #  ",
	  "    "},

	 {" ## ",
	  "  # ",
	  "  # ",
	  "    "},

	 {" ## ",
	  " ## ",
	  "    ",
	  "    "},

	 {" #  ",
	  " ## ",
	  "  # ",
	  "    "},

	 {" #  ",
	  " ## ",
	  " #  ",
	  "    "},

	 {"  # ",
	  " ## ",
	  " #  ",
	  "    "}
};
int numRots[7] = {2, 4, 4, 1, 2, 4, 2};

int vis[MAX_BOARD_SIZE][MAX_BOARD_SIZE];
int dir[4][2] = { {-1, 0}, {0, 1}, {1, 0}, {0, -1} };

// Color components in different colors
void recurse(char a[MAX_BOARD_SIZE][MAX_BOARD_SIZE], int row, int col, int color)
{
	vis[row][col] = color;
	for (int i = 0; i < 4; i++)
	{
		int nrow = row + dir[i][0]; if (nrow < 0) continue;
		int ncol = col + dir[i][1]; if (ncol < 0) continue;
		if (a[nrow][ncol] == '#' && !vis[nrow][ncol])
			recurse(a, nrow, ncol, color);
	}
}

void dropPieces(char a[MAX_BOARD_SIZE][MAX_BOARD_SIZE])
{
	// While we are erasing rows
	while (1)
	{
		// Find components
		int nextColor = 1;
		memset(vis, 0, sizeof(vis));
		for (int row = 0; row < BOARD_H + 7; row++)
			for (int col = 0; col < BOARD_W; col++)
				if (a[row][col] == '#' && !vis[row][col])
					recurse(a, row, col, nextColor++);
		
		// Drop down unsupported components
		while (1)
		{
			bool changed = false;
			for (int comp = 1; comp < nextColor; comp++)
			{
				// See if whole component is unsupported (can fall)
				int canFall = 1;
				for (int row = 0; row < BOARD_H + 7; row++)
				{
					for (int col = 0; col < BOARD_W; col++) if (vis[row][col] == comp)
					{
						if (row == 0 || (vis[row - 1][col] != 0 && vis[row - 1][col] != comp))
							{canFall = 0; break;}
					}
					if (!canFall) break;
				}
				if (!canFall) continue;
				
				// Drop each block of the component 1 square down
				for (int row = 0; row < BOARD_H + 7; row++)
				{
					for (int col = 0; col < BOARD_W; col++) if (vis[row][col] == comp)
					{
						vis[row][col] = 0; a[row][col] = ' ';
						vis[row - 1][col] = comp; a[row - 1][col] = '#';
					}
				}
				changed = true;
			}
			if (!changed) break;
		}
		
		bool changed = false;
		// Clear all complete rows
		for (int row = 0; row < BOARD_H; row++)
		{
			bool flag = true;
			for (int col = 0; col < BOARD_W; col++)
				if (a[row][col] != '#') {flag = false; break;}
			if (flag)
			{
				changed = true;
				for (int col = 0; col < BOARD_W; col++)
					a[row][col] = ' ';
			}
		}
		if (!changed) break;
	}
}

bool putFigure(char a[MAX_BOARD_SIZE][MAX_BOARD_SIZE], int fig, int rot, int col)
{
	// Copy original figure
	char f[MAX_FIG_SIZE][MAX_FIG_SIZE];
	for (int i = 0; i < MAX_FIG_SIZE; i++)
		for (int c = 0; c < MAX_FIG_SIZE; c++)
			f[i][c] = figs[fig][i][c];
	
	// Rotate it rot times clockwise
	char t[MAX_FIG_SIZE][MAX_FIG_SIZE];
	for (int r = 0; r < rot; r++)
	{
		for (int row = 0; row < MAX_FIG_SIZE; row++)
			for (int col = 0; col < MAX_FIG_SIZE; col++)
				t[col][MAX_FIG_SIZE - row - 1] = f[row][col];
		memcpy(f, t, sizeof(t));
	}

	int firstRow = MAX_FIG_SIZE, lastRow = -1;
	int firstCol = MAX_FIG_SIZE, lastCol = -1;
	for (int i = 0; i < MAX_FIG_SIZE; i++)
	{
		for (int c = 0; c < MAX_FIG_SIZE; c++) if (f[i][c] == '#')
		{
			firstRow = std::min(firstRow, i); lastRow = std::max(lastRow, i);
			firstCol = std::min(firstCol, c); lastCol = std::max(lastCol, c);
		}
	}
	
	if (col < 0 || col + lastCol - firstCol >= BOARD_W)
		return false;
	
	for (int i = lastRow; i >= firstRow; i--)
		for (int c = firstCol; c <= lastCol; c++)
			if (f[i][c] == '#') a[BOARD_H + lastRow - i][col + c - firstCol] = '#';
	
	dropPieces(a);
	for (int i = 0; i < BOARD_W; i++)
		if (a[BOARD_H][i] != ' ') return false;
	return true;
}

// <--
// Stop copying here

int eval()
{
	int score = 0;
	int have[MAX_W] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	for (int row = BOARD_H - 1; row >= 0; row--)
	{
		for (int col = 0; col < BOARD_W; col++)
		{
			if (tboard[0][row][col] == '#')
				score += heightScore * row, have[col] = 1;
			else
				score += holeScore * have[col];
		}
	}
	return score;
}

int recurse(int level, int piece, int& rot, int& pos)
{
	rot = -1; pos = -1;
	if (level == 0 || piece >= n)
		return eval();
	
	int best = -INF;
	for (int crot = 0; crot < numRots[inpFigs[piece]]; crot++)
	{
		for (int cpos = 0; cpos < BOARD_W; cpos++)
		{
			memcpy(tboard[level - 1], tboard[level], sizeof(tboard[level - 1]));
			if (!putFigure(tboard[level - 1], inpFigs[piece], crot, cpos))
				continue;
			int nrow, npos;
			int curScore = recurse(level - 1, piece + 1, nrow, npos);
			if (best < curScore) {best = curScore; rot = crot; pos = cpos;}
		}
	}
	return best;
}

void solve()
{
	unsigned sTime = clock();	
	int reached = 0, tflag = 0;

    memset(ans, 0, sizeof(ans));
    memset(board, ' ', sizeof(board));
    for (int i = 0; i < n; i++)
    {
		if (i % 10 == 0)
			if (clock() - sTime > MAX_EXEC_TIME) tflag = 1;
		memcpy(tboard[maxDepth], board, sizeof(tboard[maxDepth]));

		int rot = -1, pos = -1;
		int score = recurse(maxDepth, i, rot, pos);
		maxDepth = (!tflag && score < -300) ? 2 : 1;
		if (rot == -1 || pos == -1) break;

		reached++;
		ans[i][0] = rot; ans[i][1] = pos;
		putFigure(board, inpFigs[i], rot, pos);
//		fprintf(stderr, "For figure %d chose (%d, %d)\n", i, rot, pos);
    }
//    fprintf(stderr, "Reached figure %d out of %d.\n", reached, n);
//    fprintf(stderr, "Execution time: %.3lf seconds.\n", (double)(clock() - sTime) / CLOCKS_PER_SEC);
//    system("pause");
}


void readInput()
{
	FILE* in = stdin;
//    in = fopen("Tetris.in", "rt");
    fscanf(in, "%d", &n);
    for (int i = 0; i < n; i++)
        {fscanf(in, "%d", &inpFigs[i]); inpFigs[i]--;}
}

void writeOutput()
{
	FILE* out = stdout;
//    out = fopen("Tetris.out", "wt");
    for (int i = 0; i < n; i++)
        fprintf(out, "%d %d\n", ans[i][0], ans[i][1]);
}

int main(void)
{
    readInput();
    solve();
    writeOutput();
    return 0;
}
