/*
ID: espr1t
LANG: C++
TASK: tetris
KEYWORDS: NP, Beam Search, Simulated Annealing
*/

#include <ctime>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <algorithm>

#define MAX 131072
#define INF 1000000000

#define MAX_F 4
#define MAX_H 32
#define MAX_W 16
#define BOARD_H 20
#define BOARD_W 10

#define MAX_DEPTH 5
#define MAX_BEAMS 50
#define SAFE_SCORE 200

using namespace std;

int n;
int ans[MAX][2];
int inpFigs[MAX];
int scores[1 << BOARD_H];

// Score (less is better)
// Score for intruducing a hole
int holeScore = 100;
// Score for a square at some height
// Actual score is row * heightScore
int heightScore = 10;

char figures[7][MAX_F + 1][MAX_F + 1] = {
	 {" #  ",
	  " #  ",
	  " #  ",
	  " #  "},

	 {" ## ",
	  " #  ",
	  " #  ",
	  "    "},

	 {" ## ",
	  "  # ",
	  "  # ",
	  "    "},

	 {" ## ",
	  " ## ",
	  "    ",
	  "    "},

	 {" #  ",
	  " ## ",
	  "  # ",
	  "    "},

	 {" #  ",
	  " ## ",
	  " #  ",
	  "    "},

	 {"  # ",
	  " ## ",
	  " #  ",
	  "    "}
};
int bfigures[7][4][4];
int numRots[7] = {2, 4, 4, 1, 2, 4, 2};

int mask[MAX_W], vis[MAX_W];
int dir[4][2] = { {-1, 0}, {0, 1}, {1, 0}, {0, -1} };

void fill(int row, int col, int a[MAX_W])
{
	vis[col] |= (1 << row);
	for (int i = 0; i < 4; i++)
	{
		int nrow = row + dir[i][0]; if (nrow < 0) continue;
		int ncol = col + dir[i][1]; if (ncol < 0) continue;
		if ((a[ncol] & (1 << nrow)) && !(vis[ncol] & (1 << nrow)))
			fill(nrow, ncol, a);
	}
}

void dropPieces(int a[MAX_W])
{
	while(1)
	{
		// Clear rows if complete
		int flag = 0, cmask = a[0];
		for (int col = 1; col < BOARD_W; col++)
			cmask &= a[col];
		for (; cmask; cmask &= (cmask - 1), flag = 1)
			for (int col = 0; col < BOARD_W; col++)
				a[col] ^= (cmask ^ (cmask & (cmask - 1)));
		if (!flag) break;

		// Pieces fall down if unsupported
		while(1)
		{
			flag = 0;
			memset(vis, 0, sizeof(vis));
			for (int row = 0; row < BOARD_H + 7; row++)
			{
				for (int col = 0; col < BOARD_W; col++)
				{
					if ((a[col] & (1 << row)) && !(vis[col] & (1 << row)))
					{
						memcpy(mask, vis, sizeof(mask));
						fill(row, col, a);
						for (int i = 0; i < BOARD_W; i++)
							mask[i] ^= vis[i], vis[i] ^= mask[i];

						for (int i = 0; i < BOARD_W; i++)
							a[i] ^= mask[i];

						int howMany = MAX_H;
						for (int i = 0; i < BOARD_W; i++) if (mask[i])
						{
							int cnt = 0;
							for (; !((mask[i] >> cnt) & 1); cnt++)
								if (a[i] & (mask[i] >> (cnt + 1))) break;
							howMany = min(howMany, cnt);
						}
						if (howMany > 0) flag = 1;
						for (int i = 0; i < BOARD_W; i++) if (mask[i])
							a[i] |= (mask[i] >> howMany), vis[i] |= (mask[i] >> howMany);
					}
				}
			}
			if (!flag) break;
		}
	}
}

bool putFigure(int a[MAX_W], int fig, int rot, int col)
{
	int howMany = MAX_H;
	for (int i = 0; i < MAX_F; i++)
		mask[i] = (bfigures[fig][rot][i] << BOARD_H);
	for (int i = 0; i < MAX_F; i++) if (mask[i])
	{
		int cnt = 0;
		for (; !((mask[i] >> cnt) & 1); cnt++)
			if (a[col + i] & (mask[i] >> (cnt + 1))) break;
		howMany = min(howMany, cnt);
	}
	for (int i = 0; i < MAX_F; i++)
		a[col + i] |= (mask[i] >> howMany);
	if (a[BOARD_W]) return false;

	dropPieces(a);
	for (int i = 0; i < BOARD_W; i++)
		if (a[i] >= (1 << BOARD_H)) return false;
	return true;
}

int evalScore(int a[MAX_W])
{
	int ret = 0;
	for (int col = 0; col < BOARD_W; col++)
		ret += scores[a[col]];
	return ret;
}

struct Beam
{
	int score;
	int rot, pos;
	int board[MAX_W];

	bool operator < (const Beam& r) const
		{return score < r.score;}
};

void doWork(int board[MAX_W], int piece, int& rot, int& pos)
{
	int blen = 0, nblen = 0;
	Beam* beams = new Beam[MAX_BEAMS * 40];
	Beam* nbeams = new Beam[MAX_BEAMS * 40];

	memcpy(beams[blen].board, board, sizeof(int) * MAX_W);
	beams[blen++].score = INF;

	for (int depth = 0; depth < MAX_DEPTH; depth++)
	{
		if (blen == 0) break;
		if (piece + depth >= n) break;
		if (beams[0].score < SAFE_SCORE) break;

		nblen = 0;
		int cpiece = piece + depth;
		for (int i = 0; i < blen; i++)
		{
			for (int crot = 0; crot < numRots[inpFigs[cpiece]]; crot++)
			{
				for (int cpos = 0; cpos < BOARD_W; cpos++)
				{
					memcpy(nbeams[nblen].board, beams[i].board, sizeof(int) * MAX_W);
					if (!putFigure(nbeams[nblen].board, inpFigs[cpiece], crot, cpos))
						continue;
					nbeams[nblen].score = evalScore(nbeams[nblen].board);
					nbeams[nblen].rot = (depth == 0 ? crot : beams[i].rot);
					nbeams[nblen].pos = (depth == 0 ? cpos : beams[i].pos);
					nblen++;
				}
			}
		}
		std::sort(nbeams, nbeams + nblen);
		std::swap(beams, nbeams); blen = min(nblen, MAX_BEAMS);
	}
	rot = blen > 0 ? beams[0].rot : -1;
	pos = blen > 0 ? beams[0].pos : -1;
	delete [] beams;
	delete [] nbeams;
}

void rotate(char f[MAX_F + 1][MAX_F + 1])
{
	char t[MAX_F + 1][MAX_F + 1];
	for (int row = 0; row < MAX_F; row++)
		for (int col = 0; col < MAX_F; col++)
			t[col][MAX_F - row - 1] = f[row][col];
	memcpy(f, t, sizeof(t));
}

void precalcRotations()
{
	// Precalculate all rotations
	memset(bfigures, 0, sizeof(bfigures));
	for (int idx = 0; idx < 7; idx++)
	{
		for (int rot = 0; rot < numRots[idx]; rot++)
		{
			for (int i = 0; i < MAX_F; i++)
				for (int c = 0; c < MAX_F; c++)
					if (figures[idx][c][i] == '#')
						bfigures[idx][rot][i] |= (1 << (MAX_F - c - 1));
			while (bfigures[idx][rot][0] == 0) for (int i = 0; i < 3; i++)
			{
					bfigures[idx][rot][i] = bfigures[idx][rot][i + 1];
					bfigures[idx][rot][i + 1] = 0;
			}
        	rotate(figures[idx]);
		}
	}
}

void precalcScores()
{
    // Precalculate scores
    for (int mask = 0; mask < (1 << BOARD_H); mask++)
    {
		int flag = 0, cur = 0;
		for (int row = BOARD_H - 1; row >= 0; row--)
		{
			if (mask & (1 << row))
				cur += heightScore * row, flag = 1;
			else
				cur += holeScore * flag;
		}
		scores[mask] = cur;
	}
}

void solve()
{
	unsigned sTime = clock();

	precalcScores();
	precalcRotations();

	// Invoke solving algorithm
	int reached = 0;
	int board[MAX_W];
    memset(ans, 0, sizeof(ans));
    memset(board, 0, sizeof(board));
    for (int i = 0; i < n; i++)
    {
		int rot = -1, pos = -1;
		doWork(board, i, rot, pos);
		if (rot == -1 || pos == -1) break;

		reached++;
		ans[i][0] = rot; ans[i][1] = pos;
		putFigure(board, inpFigs[i], rot, pos);
//		if (i % 1000 == 0) fprintf(stderr, "Currently at figure %d\n", i);
    }
//    fprintf(stderr, "Reached figure %d out of %d.\n", reached, n);
//    fprintf(stderr, "Execution time: %.3lf seconds.\n", (double)(clock() - sTime) / CLOCKS_PER_SEC);
//    system("pause");
}


void readInput()
{
//	FILE* in = stdin;
//    in = fopen("Tetris.in", "rt");
    scanf("%d", &n);
    for (int i = 0; i < n; i++)
        {scanf("%d", &inpFigs[i]); inpFigs[i]--;}
}

void writeOutput()
{
//	FILE* out = stdout;
//    out = fopen("Tetris.out", "wt");
    for (int i = 0; i < n; i++)
        printf("%d %d\n", ans[i][0], ans[i][1]);
}

int main(void)
{
    readInput();
    solve();
    writeOutput();
    return 0;
}
